'use strict';

const merge = require('ember-cli-lodash-subset').merge;

module.exports = function foundation(options) {
  let defaults = {
    // @import 模块导入规则
    import: {
      path: ['node_modules/@choiceform/ui-foundation/addon/styles'],
    },

    // 导入自定义属性（CSS 变量）
    importCustomPropertiesFrom: [],

    // 保留原生 CSS 变量（高级浏览器支持）
    preserveCustomProperties: false,

    // 导入自定义媒体查询定义（Media Queries）
    importCustomMediaQueriesFrom: [],

    // 保留原生 Custom Media Queries 的定义（高级浏览器支持）
    preserveCustomeMediaQueries: false,

    // CSS 变量
    variables: {
      //   /* Typography */
      //   '--font-size': '1rem',
      //   '--font-size-xx-small': 'calc(var(--font-size-medium) * .5)',
      //   '--font-size-x-small': 'calc(var(--font-size-medium) * .625)',
      //   '--font-size-small': 'calc(var(--font-size-medium) * .75)',
      //   '--font-size-smaller': 'calc(var(--font-size-medium) * .875)',
      //   '--font-size-medium': '1rem',
      //   '--font-size-large': 'calc(var(--font-size-medium) * 1.125)',
      //   '--font-size-larger': 'calc(var(--font-size-medium) * 1.25)',
      //   '--font-size-x-large': 'calc(var(--font-size-medium) * 1.5)',
      //   '--font-size-xx-large': 'calc(var(--font-size-medium) * 2)',
      //   '--font-lighter': '200',
      //   '--font-bold': '600',
      //   /* Border Radius */
      //   '--radius-small': '2px',
      //   '--radius-medium': 'var(--cells-025)',
      //   '--radius-large': 'var(--cells-05)',
      //   '--radius-big': 'var(--cells-08)',
      //   /* Offset */
      //   '--offset': '2rem',
      //   '--cells-025': 'calc(var(--cells-2) / 8)',
      //   '--cells-05': 'calc(var(--cells-2) / 4)',
      //   '--cells-08': 'calc(var(--cells-2) / 2.5)',
      //   '--cells-1': 'calc(var(--cells-2) / 2)',
      //   '--cells-15': 'calc(var(--cells-2) * .75)',
      //   '--cells-16': 'calc(var(--cells-2) * .8)',
      //   '--cells-2': '2rem',
      //   '--cells-225': 'calc(var(--cells-2) * 1.125)',
      //   '--cells-25': 'calc(var(--cells-2) * 1.25)',
      //   '--cells-3': 'calc(var(--cells-2) * 1.5)',
      //   '--cells-35': 'calc(var(--cells-2) * 1.75)',
      //   '--cells-4': 'calc(var(--cells-2) * 2)',
      //   /* Z-index */
      //   '--z-index-1': '1',
      //   '--z-index-2': 'calc(var(--z-index-1) + 1)',
      //   '--z-index-3': 'calc(var(--z-index-2) + 1)',
      //   '--z-index-4': 'calc(var(--z-index-3) + 1)',
      //   '--z-index-5': 'calc(var(--z-index-4) + 1)',
      //   '--z-index-nav': '100',
      //   '--z-index-loading': 'calc(var(--z-index-nav) + 1)',
      //   '--z-index-modal': '1000',
      //   '--z-index-dropdown': 'calc(var(--z-index-modal) + 1)',
      //   '--z-index-notification': 'calc(var(--z-index-dropdown) + 1)',
      //   '--z-index-tooltip': 'calc(var(--z-index-notification) + 1)',
    },

    // Sassy 高级变量
    advancedVariables: {
      unresolved: 'warn',
    },

    // postcss-assets 选项
    assets: {
      basePath: 'public/',
      loadPaths: ['assets/images/'],
    },

    // postcss-calc 选项
    calc: {
      // 计算精度
      precision: 3,
      // 是否保留原始表达式（而不是求值）
      preserve: false,
      // 是否允许在 media queries 声明里求值
      mediaQueries: true,
      // 是否允许在选择符里求值
      selectors: false,
    },

    // postcss-short 选项
    short: {
      features: {
        border: {
          disable: false,
        },
        borderRadius: {
          disable: false,
        },
        color: {
          disable: false,
        },
        fontSize: {
          disable: false,
        },
        fontWeights: {
          disable: false,
        },
        position: {
          disable: false,
        },
        size: {
          disable: false,
        },
        spacing: {
          disable: false,
        },
      },
      skip: '*',
    },

    // postcss-clear 选项
    clear: {
      display: 'block',
    },
  };

  options = merge({}, defaults, options);

  return {
    babel: {
      plugins: [['@babel/plugin-proposal-object-rest-spread', {}]],
    },

    sourcemaps: {
      enabled: process.env.EMBER_ENV === 'development',
      extensions: ['css', 'js'],
    },

    // 本地模块化样式选项
    cssModules: {
      // PostCSS 选项
      postcssOptions: {
        // 使用 PostCSS 生成样式的 sourcemaps，用于浏览器内对应模块的跳转（推荐）
        map: process.env.EMBER_ENV === 'development',
      },

      // PostCSS 插件，它们的选项在本文件的前面
      plugins: {
        before: [
          require('postcss-import')(options.import),
          require('postcss-nesting'),
        ],
        postprocess: [
          require('postcss-extend'),
          require('postcss-custom-properties')({
            preserve: options.preserveCustomProperties,
            importFrom: options.importCustomPropertiesFrom,
          }),
          require('postcss-advanced-variables')(options.advancedVariables),
          require('postcss-custom-media')({
            preserve: options.preserveCustomeMedia,
            importFrom: options.importCustomMediaFrom,
          }),
          require('postcss-media-minmax'),
          require('postcss-color-mod-function')({
            transformVars: true,
            unresolved: 'warn',
          }),
          require('postcss-image-set-function')({
            preserve: options.preserve,
          }),
          require('postcss-object-fit-images'),
          require('postcss-calc')(options.calc),
          require('postcss-assets')(options.assets),
          require('postcss-fallback'),
          require('postcss-mobile-hover'),
          require('postcss-short')(options.short),
          require('postcss-clearfix')(options.clear),
          require('postcss-merge-rules'),
          require('postcss-discard-empty'),
          require('postcss-discard-comments'),
          require('postcss-discard-duplicates'),
        ],
      },
    },

    // 不依赖 development 环境下生成的 tests
    tests: process.env.EMBER_ENV !== 'development',

    // 是否默认包含 device.js
    useDeviceJS: false,

    // 是否默认包含 normalize.css（推荐）
    useNormalizeCSS: false,

    // 是否默认包含 object-fit 的 polyfill
    // Usage: https://github.com/bfred-it/object-fit-images#usage
    useObjectFitImagesPolyfill: false,

    // 是否默认导入 SVG 图标（'survey-client' 将导入专门为 os-client 设计的图标集），`false` 则不导入任何图标
    importSVGIcon: false,

    // 是否将应用的环境选项作为 <meta> 标签注入到 HTML 文档中，否则编译到最终生成的 js 代码中
    storeConfigInMeta: process.env.EMBER_ENV === 'test',

    // 是否在产品环境下禁用 Ember Inspector
    disableInspectorInProductionMode: true,
  };
};
