'use strict';

const fs = require('fs');
const path = require('path');
const map = require('broccoli-stew').map;
const npmPackage = require('./package.json');
const Funnel = require('broccoli-funnel');
const mergeTrees = require('broccoli-merge-trees');
const merge = require('ember-cli-lodash-subset').merge;

module.exports = {
  name: require('./package').name,

  config() {
    return {
      [this.name]: {
        version: npmPackage.version,
      },
    };
  },

  included(parent) {
    this.options = merge({}, this.options, parent.options);
    this._super.included.apply(this, arguments);

    if (this.options.useDeviceJS) {
      this.import('vendor/device.js/device.umd.js');
    }

    if (this.options.useObjectFitImagesPolyfill) {
      this.import('node_modules/object-fit-images/dist/ofi.js');
    }

    if ('test' !== process.env.EMBER_ENV && this.options.useNormalizeCSS) {
      this.import('node_modules/normalize.css/normalize.css', {
        prepend: true,
      });
    }
  },

  treeForVendor() {
    const defaultTree = this._super.treeForAddon.apply(this, arguments);

    if (this.options.useDeviceJS) {
      const deviceJSTree = this._customizedFastbootWrapper();
      return defaultTree
        ? new mergeTrees([defaultTree, deviceJSTree])
        : deviceJSTree;
    }

    return defaultTree;
  },

  _customizedFastbootWrapper() {
    const deviceJSOpts = {
      srcDir: 'dist',
      include: ['device.umd.js'],
      destDir: 'device.js',
    };
    const wrapperFunc = content =>
      `if (typeof FastBoot === 'undefined') {\n${content}\n` +
      `window.device = device.device; device.addClasses(document.documentElement);\n}`;
    return map(new Funnel('node_modules/device.js', deviceJSOpts), wrapperFunc);
  },

  contentFor(type, config, content) {
    if ('head' === type) {
      return this._contentForHead.call(this, config, content);
    }

    if ('body' === type) {
      return this._contentForBody.call(this, config, content);
    }
  },

  _contentForHead(/* config, content */) {
    const content = [];

    if (
      this.options.disableInspectorInProductionMode &&
      this.app.env === 'production'
    ) {
      content.push(`<script>window.NO_EMBER_DEBUG = true;</script>`);
    }

    return content.join('\n');
  },

  _contentForBody(/* config, content */) {
    const content = [];

    if (this.options.importSVGIcon) {
      let filePath = this._filePathForSVGIcon(this.options.importSVGIcon);
      content.push(fs.readFileSync(filePath, 'utf-8').replace(/\n\r?/g, ''));
    }

    return content.join('\n');
  },

  _filePathForSVGIcon(filename) {
    const resolve = require('resolve');
    const basePath = this.project.pkg.name.includes(this.name)
      ? this.project.root
      : path.dirname(resolve.sync(this.name, { basedir: this.project.root }));

    return path.resolve(basePath, `addon/${filename}-icons.html`);
  },
};
